package com.chelvc.cloud.vehicle.server.service.impl;

import cn.hutool.core.map.MapBuilder;
import cn.hutool.core.text.CharSequenceUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.chelvc.cloud.vehicle.api.constant.PromotionScopeType;
import com.chelvc.cloud.vehicle.api.constant.PromotionStatus;
import com.chelvc.cloud.vehicle.api.param.BasePromotionQueryParam;
import com.chelvc.cloud.vehicle.server.entity.BasePromotion;
import com.chelvc.cloud.vehicle.server.entity.PromotionGoods;
import com.chelvc.cloud.vehicle.server.service.AbstractPromotionService;
import com.chelvc.cloud.vehicle.server.service.PromotionGoodsService;
import com.chelvc.cloud.vehicle.server.util.PromotionUtils;
import com.chelvc.framework.base.exception.ResourceUnavailableException;
import com.chelvc.framework.common.util.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationEventPublisher;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;

/**
 * 抽象通用促销服务业务操作实现
 *
 * @author qizai
 * @date 2023/11/12
 */
public abstract class AbstractPromotionServiceImpl<M extends BaseMapper<T>, T extends BasePromotion> extends ServiceImpl<M, T> implements AbstractPromotionService<T> {

    /**
     * 促销商品
     */
    @Autowired
    private PromotionGoodsService promotionGoodsService;

    @Autowired
    private ApplicationEventPublisher applicationEventPublisher;

    /**
     * 通用促销保存
     * 调用顺序:
     * 1. initPromotion 初始化促销信息
     * 2. checkPromotion 检查促销参数
     * 3. save 保存促销信息
     * 4. updatePromotionGoods 更新促销商品信息
     * 5。 updateEsGoodsIndex 更新商品索引促销信息
     *
     * @param promotion 促销信息
     * @return 是否保存成功
     */
    @Override
    @Transactional(rollbackFor = {Exception.class})
    public boolean savePromotion(T promotion) {
        this.initPromotion(promotion);
        this.checkPromotion(promotion);
        boolean save = this.save(promotion);
        if (this.updatePromotionsGoods(promotion)) {
            this.updateEsGoodsIndex(promotion);
        }
        return save;
    }

    /**
     * 通用促销更新
     * 调用顺序:
     * 1. checkStatus 检查促销状态
     * 2. checkPromotions 检查促销参数
     * 3. saveOrUpdate 保存促销信息
     * 4. updatePromotionGoods 更新促销商品信息
     * 5. updateEsGoodsIndex 更新商品索引促销信息
     *
     * @param promotions 促销信息
     * @return 是否更新成功
     */
    @Override
    @Transactional(rollbackFor = {Exception.class})
    public boolean updatePromotion(T promotions) {
        this.checkStatus(promotions);
        this.checkPromotion(promotions);
        boolean save = this.saveOrUpdate(promotions);
        if (this.updatePromotionsGoods(promotions)) {
            this.updateEsGoodsIndex(promotions);
        }
        return save;
    }

    /**
     * 更新促销状态
     * 如果要更新促销状态为关闭，startTime和endTime置为空即可
     *
     * @param ids       促销id集合
     * @param startTime 开始时间
     * @param endTime   结束时间
     * @return 是否更新成功
     */
    @Override
    @Transactional(rollbackFor = {Exception.class})
    public boolean updateStatus(List<Long> ids, Date startTime, Date endTime) {
        List<T> promotionsList = this.list(new QueryWrapper<T>().in("id", ids));
        for (T t : promotionsList) {
            if (startTime != null && endTime != null) {
                t.setStartTime(startTime);
                t.setEndTime(endTime);
            } else {
                t.setStartTime(null);
                t.setEndTime(null);
            }
            this.checkStatus(t);
            this.updatePromotionsGoods(t);
            this.updateEsGoodsIndex(t);
        }
        if (startTime != null && endTime != null) {
            return this.update(new UpdateWrapper<T>().in("id", ids)
                    .set("start_time", startTime).set("end_time", endTime));
        } else {
            return this.update(new UpdateWrapper<T>().in("id", ids)
                    .set("start_time", null).set("end_time", null));
        }
    }

    /**
     * 移除促销活动
     *
     * @param ids 促销活动id集合
     * @return 是否移除成功
     */
    @Override
    @Transactional(rollbackFor = {Exception.class})
    public boolean removePromotion(List<Long> ids) {
        for (Long id : ids) {
            T promotions = this.getById(id);
            this.checkStatus(promotions);
            promotions.setStartTime(null);
            promotions.setEndTime(null);
            this.updateEsGoodsIndex(promotions);
        }
        this.promotionGoodsService.deletePromotionGoods(ids);
        return this.removeByIds(ids);
    }

    /**
     * 初始化促销字段
     *
     * @param promotion 促销实体
     */
    @Override
    public void initPromotion(T promotion) {
        if (StringUtils.isEmpty(promotion.getScopeType())) {
            promotion.setScopeType(PromotionScopeType.ALL);
        }
    }

    /**
     * 检查促销参数
     *
     * @param promotion 促销实体
     */
    @Override
    public void checkPromotion(T promotion) {
        PromotionUtils.checkPromotionTime(promotion.getStartTime(), promotion.getEndTime());
    }

    /**
     * 检查促销状态
     *
     * @param promotion 促销实体
     */
    @Override
    public void checkStatus(T promotion) {
        T byId = this.getById(promotion.getId());
        if (byId == null) {
            throw new ResourceUnavailableException("当前促销活动不存在");
        }
    }

    /**
     * 更新促销商品信息
     *
     * @param promotion 促销实体
     * @return
     */
    @Override
    @Transactional(rollbackFor = {Exception.class})
    public boolean updatePromotionsGoods(T promotion) {
        if (promotion.getStartTime() == null && promotion.getEndTime() == null) {
            this.promotionGoodsService.deletePromotionGoods(Collections.singletonList(promotion.getId()));
            return true;
        }
        boolean result = true;
        if (CharSequenceUtil.equalsAny(promotion.getScopeType().name(), PromotionScopeType.ALL.name(),
                PromotionScopeType.PORTION_GOODS_CATEGORY.name())) {
            PromotionGoods promotionGoods = new PromotionGoods();
            promotionGoods.setScopeId(promotion.getScopeId());
            promotionGoods.setScopeType(promotion.getScopeType());
            promotionGoods.setPromotionId(promotion.getId());
            promotionGoods.setMerchantId(promotion.getMerchantId());
            promotionGoods.setMerchantName(promotion.getMerchantName());
            promotionGoods.setStartTime(promotion.getStartTime());
            promotionGoods.setEndTime(promotion.getEndTime());
            promotionGoods.setPromotionType(this.getPromotionType());
            promotionGoods.setTitle(promotion.getPromotionName());
            this.promotionGoodsService.deletePromotionGoods(Collections.singletonList(promotion.getId()));
            result = this.promotionGoodsService.save(promotionGoods);
        }
        return result;
    }

    /**
     * 更新促销信息到商品索引
     *
     * @param promotion 促销实体
     */
    @Override
    @Transactional(rollbackFor = {Exception.class})
    public void updateEsGoodsIndex(T promotion) {
        if (promotion.getStartTime() == null && promotion.getEndTime() == null) {
            Map<Object, Object> build = MapBuilder.create().put("promotionKey", this.getPromotionType()
                    + "-" + promotion.getId()).put("scopeId", promotion.getScopeId()).build();
            //删除商品促销消息
//            applicationEventPublisher.publishEvent(new TransactionCommitSendMQEvent("删除商品促销事件", rocketmqCustomProperties.getGoodsTopic(), GoodsTagsEnum.DELETE_GOODS_INDEX_PROMOTIONS.name(), JSONUtil.toJsonStr(build)));
        } else {

            String esPromotionKey = this.getPromotionType().name() + "-" + promotion.getId();
            Map<String, Object> map = new HashMap<>();
            // es促销key
            map.put("esPromotionKey", esPromotionKey);
            // 促销类型全路径名
            map.put("promotionsType", promotion.getClass().getName());
            // 促销实体
            map.put("promotions", promotion);
//            applicationEventPublisher.publishEvent(new TransactionCommitSendMQEvent("更新商品索引促销事件", rocketmqCustomProperties.getGoodsTopic(), GoodsTagsEnum.UPDATE_GOODS_INDEX_PROMOTIONS.name(), JSONUtil.toJsonStr(map)));
        }
    }

    @Override
    public <S extends BasePromotionQueryParam> List<T> listFindAll(S queryParam) {
        QueryWrapper<T> queryWrapper = this.baseQueryWrapper(queryParam);

        if (CharSequenceUtil.isNotEmpty(queryParam.getPromotionStatus())) {
            queryWrapper.and(i -> {
                for (String status : queryParam.getPromotionStatus().split(",")) {
                    i.or(PromotionUtils.queryPromotionStatus(PromotionStatus.valueOf(status)));
                }
            });
        }
        return this.list(queryWrapper);
    }

    public <T> QueryWrapper<T> baseQueryWrapper(BasePromotionQueryParam queryParam) {
        QueryWrapper<T> queryWrapper = new QueryWrapper<>();

        if (!Objects.isNull(queryParam.getId())) {
            queryWrapper.eq("id", queryParam.getId());
        }
        if (queryParam.getStartTime() != null) {
            queryWrapper.ge("start_time", queryParam.getStartTime());
        }
        if (queryParam.getEndTime() != null) {
            queryWrapper.le("end_time", queryParam.getEndTime());
        }
        if (!Objects.isNull(queryParam.getScopeType())) {
            queryWrapper.eq("scope_type", queryParam.getScopeType());
        }
        if (CharSequenceUtil.isNotEmpty(queryParam.getMerchantId())) {
            if ("platform".equals(queryParam.getMerchantId())) {
                queryWrapper.ne("merchant_id", PromotionUtils.PLATFORM_ID);
            } else {
                queryWrapper.in("merchant_id", Arrays.asList(queryParam.getMerchantId().split(",")));
            }
        }
        queryWrapper.eq("deleted", false);
        return queryWrapper;
    }
}
