package com.chelvc.cloud.vehicle.server.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.text.CharSequenceUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.extension.conditions.query.LambdaQueryChainWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.chelvc.cloud.vehicle.api.constant.*;
import com.chelvc.cloud.vehicle.api.dto.CouponDTO;
import com.chelvc.cloud.vehicle.api.param.CouponModifyParam;
import com.chelvc.cloud.vehicle.api.param.CouponPagingParam;
import com.chelvc.cloud.vehicle.api.param.FullDiscountQueryParam;
import com.chelvc.cloud.vehicle.api.param.PromotionGoodsQueryParam;
import com.chelvc.cloud.vehicle.server.copier.CouponCopier;
import com.chelvc.cloud.vehicle.server.dao.CouponMapper;
import com.chelvc.cloud.vehicle.server.dao.GoodsMapper;
import com.chelvc.cloud.vehicle.server.dao.UserCouponMapper;
import com.chelvc.cloud.vehicle.server.entity.Coupon;
import com.chelvc.cloud.vehicle.server.entity.FullDiscount;
import com.chelvc.cloud.vehicle.server.entity.Goods;
import com.chelvc.cloud.vehicle.server.entity.PromotionGoods;
import com.chelvc.cloud.vehicle.server.service.*;
import com.chelvc.cloud.vehicle.server.util.PromotionUtils;
import com.chelvc.framework.base.exception.ResourceUnavailableException;
import com.chelvc.framework.base.util.ResourceUtils;
import com.chelvc.framework.common.model.Pagination;
import com.chelvc.framework.common.util.DateUtils;
import com.chelvc.framework.database.util.PagingUtils;
import lombok.NonNull;
import lombok.RequiredArgsConstructor;
import org.apache.dubbo.config.annotation.DubboService;
import org.apache.poi.ss.formula.functions.T;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.*;
import java.util.function.Consumer;
import java.util.stream.Collectors;

/**
 * 优惠券业务操作实现
 *
 * @author Woody
 * @date 2023/7/17
 */
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
@DubboService(interfaceClass = com.chelvc.cloud.vehicle.api.service.CouponService.class)
public class CouponServiceImpl extends AbstractPromotionServiceImpl<CouponMapper, Coupon> implements CouponService,
        com.chelvc.cloud.vehicle.api.service.CouponService {

    private final CouponActivityItemService couponActivityItemService;
    private final PromotionGoodsService promotionGoodsService;
    private final FullDiscountService fullDiscountService;
    private final GoodsMapper goodsMapper;
    private final UserCouponMapper userCouponMapper;

    @Override
    public List<CouponDTO> listUserGoodsActiveCoupons(@NonNull Long userId, Collection<Long> goodsCouponIds) {
        return this.baseMapper.listUserGoodsActiveCoupons(userId, goodsCouponIds);
    }

    @Override
    public Long addCoupon(@NonNull CouponModifyParam param) {
        Coupon coupon = CouponCopier.INSTANCE.copying(param);
        this.save(coupon);
        return coupon.getId();
    }

    @Override
    public void updateCoupon(@NonNull Long id, @NonNull CouponModifyParam param) {
        Coupon coupon = ResourceUtils.required(this.getById(id), "优惠券不存在");
        CouponCopier.INSTANCE.copying(param, coupon);
        // 检查促销状态
        this.checkStatus(coupon);
        // 检查促销参数
        this.checkPromotion(coupon);
        // 修改优惠券信息
        this.updateById(coupon);
        // 更新促销商品信息
        this.updatePromotionsGoods(coupon);
    }

    @Override
    public void deleteCoupon(@NonNull Long id) {
        ResourceUtils.required(this.getById(id), "商品优惠券不存在");
        this.baseMapper.logicDelete(id);
    }

    @Override
    public CouponDTO getCoupon(@NonNull Long id) {
        Coupon coupon = ResourceUtils.required(this.getById(id), "优惠券不存在");
        List<PromotionGoods> promotionGoods = this.promotionGoodsService
                .listFindAll(PromotionGoodsQueryParam.builder().promotionId(id).build());
        if (CollectionUtil.isNotEmpty(promotionGoods)) {
            coupon.setPromotionGoodsList(promotionGoods);
        }
        return this.convert(coupon);
    }

    @Override
    public Long saveCoupon(@NonNull CouponModifyParam param) {
        Coupon coupon = CouponCopier.INSTANCE.copying(param);
        if (PromotionUtils.PLATFORM_ID.equals(coupon.getMerchantId())) {
            coupon.setMerchantName(PromotionUtils.PLATFORM_NAME);
        }
        // 初始化促销信息
        this.initPromotion(coupon);
        // 检查促销参数
        this.checkPromotion(coupon);
        // 保存优惠券信息
        this.save(coupon);
        // 更新促销商品信息
        this.updatePromotionsGoods(coupon);
        return coupon.getId();
    }

    @Override
    public Pagination<CouponDTO> getCouponPaging(@NonNull CouponPagingParam param) {
        Long merchantId = param.getMerchantId();
        if (Objects.isNull(merchantId)) {
            merchantId = PromotionUtils.PLATFORM_ID;
        }
        // 查询优惠券列表
        LambdaQueryChainWrapper<Coupon> lambdaQueryChainWrapper = this.lambdaQuery()
                .eq(Coupon::getMerchantId, merchantId)
                .eq(Objects.nonNull(param.getCouponClaimType()), Coupon::getClaimType, param.getCouponClaimType())
                .eq(Objects.nonNull(param.getPromotionStatus()), Coupon::getStatus, param.getPromotionStatus())
                .eq(Objects.nonNull(param.getCouponType()), Coupon::getType, param.getCouponType())
                .ge(Objects.nonNull(param.getStartTime()), Coupon::getStartTime, param.getStartTime())
                .le(Objects.nonNull(param.getEndTime()), Coupon::getStartTime, param.getEndTime())
                .eq(Coupon::getDeleted, false);
        if (Objects.nonNull(param.getPromotionStatus())) {
            lambdaQueryChainWrapper.nested(this.queryPromotionStatus(param.getPromotionStatus()));
        }
        Page<Coupon> page = lambdaQueryChainWrapper
                .like(!StringUtils.isEmpty(param.getKeyword()), Coupon::getName, param.getKeyword())
                .orderByDesc(Coupon::getCreateTime).page(PagingUtils.convert(param.getPaging()));
        List<Coupon> records = page.getRecords();
        if (CollectionUtils.isEmpty(records)) {
            return Pagination.empty();
        }

        // 构建优惠券信息
        List<CouponDTO> clients = records.stream().map(this::convert).collect(Collectors.toList());
        return PagingUtils.convert(page, clients);
    }

    @Override
    public void updateCouponStatus(@NonNull CouponModifyParam param) {
        this.updateStatus(param.getIds(), param.getStartTime(), param.getEndTime());
    }

    @Override
    public void batchDeleteCoupon(@NonNull List<Long> ids) {
        // 关闭用户优惠券信息
        this.userCouponMapper.updateUserCouponStatus(ids, UserCouponStatus.CLOSED);
        // 删除优惠券关联优惠券活动
        this.couponActivityItemService.removeByCouponId(ids);
        super.removePromotion(ids);
    }

    @Override
    public Pagination<CouponDTO> getMerchantCouponPaging(@NonNull Long merchantId, @NonNull CouponPagingParam param) {
        // 查询优惠券列表
        Page<Coupon> page = this.lambdaQuery()
                .eq(Coupon::getMerchantId, merchantId)
                .orderByDesc(Coupon::getCreateTime).page(PagingUtils.convert(param.getPaging()));
        List<Coupon> records = page.getRecords();
        if (CollectionUtils.isEmpty(records)) {
            return Pagination.empty();
        }

        // 构建优惠券信息
        List<CouponDTO> clients = records.stream().map(this::convert).collect(Collectors.toList());
        return PagingUtils.convert(page, clients);
    }

    @Override
    public void updateCouponReceivedNum(Long couponId, Integer receiveNum) {
        Coupon coupon = this.getById(couponId);
        if (Objects.isNull(coupon)) {
            throw new ResourceUnavailableException("当前优惠券不存在");
        }
        int row = this.baseMapper.updateCouponReceivedNum(couponId, receiveNum, coupon.getPublishNum());
        if (row < 1) {
            throw new ResourceUnavailableException("修改优惠券已被领取数量失败！");
        }
    }

    @Override
    public boolean updateStatus(List<Long> ids, Date startTime, Date endTime) {
        List<Coupon> coupons = this.lambdaQuery()
                .in(Coupon::getId, ids)
                .eq(Coupon::getRangeDayType, CouponRangeDay.DYNAMICTIME).list();
        if (!coupons.isEmpty()) {
            LambdaUpdateWrapper<Coupon> updateWrapper = new LambdaUpdateWrapper<>();
            updateWrapper.in(Coupon::getId, coupons.stream().map(Coupon::getId).collect(Collectors.toList()));
            updateWrapper.set(Coupon::getEffectiveDays, 0);
            updateWrapper.set(Coupon::getPromotionStatus, PromotionStatus.CLOSE);
            this.update(updateWrapper);
        }

        // 关闭优惠券，删除相关会员优惠券和券活动
        if (startTime == null && endTime == null) {
            //关闭用户优惠券信息
            this.userCouponMapper.updateUserCouponStatus(ids, UserCouponStatus.CLOSED);
            //删除优惠券关联优惠券活动
            this.couponActivityItemService.removeByCouponId(ids);
        }
        return super.updateStatus(ids, startTime, endTime);
    }

    @Override
    public void initPromotion(Coupon coupon) {
        coupon.setUsedNum(0);
        coupon.setReceivedNum(0);
    }

    @Override
    public void checkPromotion(Coupon coupon) {
        if (coupon.getRangeDayType() == null) {
            super.checkPromotion(coupon);
        }
        //如果发行数量是0则判断领取限制数量
        if (coupon.getPublishNum() != 0 && coupon.getLimitNum() > coupon.getPublishNum()) {
            throw new ResourceUnavailableException("领取限制数量超出发行数量");
        }
        //打折优惠券大于10折
        boolean discountCoupon = (coupon.getType().equals(CouponType.DISCOUNT)
                && (coupon.getDiscount() < 0 || coupon.getDiscount() > 10));
        if (discountCoupon) {
            throw new ResourceUnavailableException("优惠券折扣必须小于10且大于0");
        }

        switch (coupon.getRangeDayType()) {
            case FIXEDTIME:
                //如果优惠券为固定时间，则开始结束时间不能为空
                if (coupon.getEndTime() == null || coupon.getStartTime() == null) {
                    throw new ResourceUnavailableException("活动起始时间和活动结束时间不能为空");
                }
                long nowTime = DateUtils.timestamp();
                //固定时间的优惠券不能小于当前时间
                if (coupon.getEndTime().getTime() < nowTime) {
                    throw new ResourceUnavailableException("活动结束时间不能小于当前时间");
                }
                break;
            case DYNAMICTIME:
                //动态时间的优惠券不能小于当前时间
                if (coupon.getEffectiveDays() == null || coupon.getEffectiveDays() < 0) {
                    throw new ResourceUnavailableException("活动结束时间不能小于当前时间");
                }
                break;
        }

        this.checkCouponScope(coupon);
    }

    @Override
    public void checkStatus(Coupon coupon) {
        super.checkStatus(coupon);
        FullDiscountQueryParam queryParam = new FullDiscountQueryParam();
        queryParam.setCouponFlag(true);
        queryParam.setCouponId(coupon.getId());
        queryParam.setPromotionStatus(PromotionStatus.START.name());
        List<FullDiscount> fullDiscounts = fullDiscountService.listFindAll(queryParam);
        if (fullDiscounts != null && !fullDiscounts.isEmpty()) {
            throw new ResourceUnavailableException("当前优惠券参与了促销活动【" + fullDiscounts.get(0).getPromotionName()
                    + "】不能进行编辑删除操作");
        }
    }

    @Override
    public boolean updatePromotionsGoods(Coupon coupon) {
        boolean result = super.updatePromotionsGoods(coupon);
        if (!PromotionStatus.CLOSE.equals(coupon.getPromotionStatus()) &&
                PromotionScopeType.PORTION_GOODS.equals(coupon.getScopeType())) {
            this.promotionGoodsService.deletePromotionGoods(Collections.singletonList(coupon.getId()));
            List<PromotionGoods> promotionGoodsList = PromotionUtils.promotionGoodsInit(coupon.getPromotionGoodsList(),
                    coupon, this.getPromotionType());
            for (PromotionGoods promotionGoods : promotionGoodsList) {
                promotionGoods.setMerchantId(coupon.getMerchantId());
                promotionGoods.setMerchantName(coupon.getMerchantName());
            }
            //促销活动商品更新
            result = this.promotionGoodsService.saveBatch(promotionGoodsList);
        }
        return result;
    }

    @Override
    public PromotionType getPromotionType() {
        return PromotionType.COUPON;
    }

    /**
     * 检查优惠券范围
     *
     * @param coupon 检查的优惠券对象
     */
    private void checkCouponScope(Coupon coupon) {
        boolean portionGoodsScope = (coupon.getScopeType().equals(PromotionScopeType.PORTION_GOODS)
                && (coupon.getPromotionGoodsList() == null || coupon.getPromotionGoodsList().isEmpty()));
        if (portionGoodsScope) {
            throw new ResourceUnavailableException("当前关联范围类型为指定商品时，商品列表不能为空");
        } else if (coupon.getScopeType().equals(PromotionScopeType.PORTION_GOODS)
                && CharSequenceUtil.isEmpty(coupon.getScopeId())) {
            throw new ResourceUnavailableException("当前关联范围类型为指定商品时，范围关联的id不能为空");
        } else if (coupon.getScopeType().equals(PromotionScopeType.PORTION_GOODS_CATEGORY)
                && CharSequenceUtil.isEmpty(coupon.getScopeId())) {
            throw new ResourceUnavailableException("当前关联范围类型为部分商品分类时，范围关联的id不能为空");
        } else if (coupon.getScopeType().equals(PromotionScopeType.PORTION_SHOP_CATEGORY)
                && CharSequenceUtil.isEmpty(coupon.getScopeId())) {
            throw new ResourceUnavailableException("当前关联范围类型为部分店铺分类时，范围关联的id不能为空");
        }

        if (coupon.getScopeType().equals(PromotionScopeType.PORTION_GOODS)) {
            this.checkCouponPortionGoods(coupon);
        }
    }

    /**
     * 检查指定商品
     *
     * @param coupon 优惠券信息
     */
    private void checkCouponPortionGoods(Coupon coupon) {
        String[] split = coupon.getScopeId().split(",");
        if (split.length == 0) {
            throw new ResourceUnavailableException("指定商品范围关联id不能为空");
        }
        for (String id : split) {
            Goods goods = this.goodsMapper.selectById(id);
            if (goods == null) {
                throw new ResourceUnavailableException("商品不存在");
            }
        }
    }

    /**
     * 转换优惠券信息
     *
     * @param coupon 优惠券实例
     * @return 优惠券信息
     */
    private CouponDTO convert(Coupon coupon) {
        return CouponCopier.INSTANCE.copying(coupon);
    }

    private <T> Consumer<LambdaQueryWrapper<Coupon>> queryPromotionStatus(PromotionStatus promotionStatus) {
        switch (promotionStatus) {
            case NEW:
                return (LambdaQueryWrapper<Coupon> t) -> t.nested(i -> i.gt(Coupon::getStartTime, new Date())
                        .gt(Coupon::getEndTime, new Date()));
            case START:
                return (LambdaQueryWrapper<Coupon> t) -> t.nested(i -> i.le(Coupon::getStartTime, new Date())
                        .ge(Coupon::getEndTime, new Date()));
            case END:
                return (LambdaQueryWrapper<Coupon> t) -> t.nested(i -> i.lt(Coupon::getStartTime, new Date())
                        .lt(Coupon::getEndTime, new Date()));
            case CLOSE:
                return (LambdaQueryWrapper<Coupon> t) -> t.nested(i -> i.isNull(Coupon::getStartTime)
                        .isNull(Coupon::getEndTime));
            default:
                return null;
        }
    }
}