package com.chelvc.cloud.vehicle.server.util;

import cn.hutool.core.text.CharSequenceUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.chelvc.cloud.vehicle.api.constant.PromotionStatus;
import com.chelvc.cloud.vehicle.api.constant.PromotionType;
import com.chelvc.cloud.vehicle.server.entity.BasePromotion;
import com.chelvc.cloud.vehicle.server.entity.PromotionGoods;
import com.chelvc.framework.common.util.DateUtils;
import lombok.extern.slf4j.Slf4j;

import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.function.Consumer;

/**
 * 优惠活动通用验证类
 *
 * @author qizai
 * @date 2023/10/16
 */
@Slf4j
public class PromotionUtils {

    public static final String START_TIME_COLUMN = "start_time";
    public static final String END_TIME_COLUMN = "end_time";
    public static final Long PLATFORM_ID = 0L;
    public static final String PLATFORM_NAME = "platform";

    /**
     * 参数验证
     * 1、活动起始时间必须大于当前时间
     * 2、验证活动开始时间是否大于活动结束时间
     *
     * @param startTime 活动开始时间
     * @param endTime   活动结束时间
     */
    public static void checkPromotionTime(Date startTime, Date endTime) {
        if (startTime == null) {
            throw new RuntimeException("活动起始时间不能为空");
        }

        Date now = DateUtils.today();

        //如果活动起始时间小于现在时间
        if (now.after(startTime)) {
            throw new RuntimeException("活动起始时间不能小于当前时间");
        }
        //如果活动结束时间小于现在时间
        if (endTime != null && now.after(endTime)) {
            throw new RuntimeException("活动结束时间不能小于当前时间");
        }
        //开始时间不能大于结束时间
        if (endTime != null && startTime.after(endTime)) {
            throw new RuntimeException("活动起始时间必须大于结束时间");
        }
    }

    /**
     * 促销商品入库前填充
     *
     * @param originList 原促销商品列表
     * @param promotion  促销信息
     * @return 促销商品列表
     */
    public static List<PromotionGoods> promotionGoodsInit(List<PromotionGoods> originList, BasePromotion promotion,
                                                          PromotionType promotionType) {
        if (originList != null) {
            //本次促销商品入库
            for (PromotionGoods promotionGoods : originList) {
                promotionGoods.setPromotionId(promotion.getId());
                if (Objects.isNull(promotionGoods.getMerchantId())) {
                    promotionGoods.setMerchantId(promotion.getMerchantId());
                }
                if (CharSequenceUtil.isEmpty(promotionGoods.getMerchantName())) {
                    promotionGoods.setMerchantName(promotion.getMerchantName());
                }
                promotionGoods.setTitle(promotion.getPromotionName());
                // 如果是秒杀活动保留原时间
                if (promotionGoods.getStartTime() == null || !PromotionType.SECKILL.equals(promotionType)) {
                    promotionGoods.setStartTime(promotion.getStartTime());
                }
                if (promotionGoods.getStartTime() == null || !PromotionType.SECKILL.equals(promotionType)) {
                    promotionGoods.setEndTime(promotion.getEndTime());
                }
                promotionGoods.setPromotionType(promotionType);
                promotionGoods.setNum(0);
                promotionGoods.setDeleted(promotion.getDeleted());
            }
        }
        return originList;
    }

    public static <T> Consumer<QueryWrapper<T>> queryPromotionStatus(PromotionStatus promotionStatus) {
        switch (promotionStatus) {
            case NEW:
                return (QueryWrapper<T> t) -> t.nested(i -> i.gt(START_TIME_COLUMN, new Date())
                        .gt(END_TIME_COLUMN, new Date()));
            case START:
                return (QueryWrapper<T> t) -> t.nested(i -> i.le(START_TIME_COLUMN, new Date())
                        .ge(END_TIME_COLUMN, new Date()));
            case END:
                return (QueryWrapper<T> t) -> t.nested(i -> i.lt(START_TIME_COLUMN, new Date())
                        .lt(END_TIME_COLUMN, new Date()));
            case CLOSE:
                return (QueryWrapper<T> t) -> t.nested(i -> i.isNull(START_TIME_COLUMN).isNull(END_TIME_COLUMN));
            default:
                return null;
        }
    }
}
